@@ -1,10 +1,9 @@
 # python imports
 import datetime
 import re
-import random
 
 # django imports
-from django import forms
+from django import template
 from django.conf import settings
 from django.contrib.auth.models import User
 from django.contrib.contenttypes import generic
@@ -12,20 +11,13 @@ from django.contrib.contenttypes.models import ContentType
 from django.core.cache import cache
 from django.core.urlresolvers import reverse
 from django.db import models
-from django.db.models.signals import post_syncdb
 from django.template import RequestContext
 from django.template.loader import render_to_string
 from django.utils import translation
 from django.utils.translation import ugettext_lazy as _
 
 # tagging imports
-import tagging.utils
 from tagging import fields
-from tagging.forms import TagField
-
-# portlets imports
-from portlets.models import Portlet
-from portlets.utils import register_portlet
 
 # workflows imports
 import workflows.utils
@@ -41,18 +33,21 @@ from permissions.models import Role
 # lfc imports
 import lfc.utils
 from lfc.fields.thumbs import ImageWithThumbsField
-from lfc.fields.autocomplete import AutoCompleteTagInput
 from lfc.managers import BaseContentManager
 from lfc.settings import ALLOW_COMMENTS_CHOICES
 from lfc.settings import ALLOW_COMMENTS_DEFAULT
 from lfc.settings import ALLOW_COMMENTS_TRUE
 from lfc.settings import LANGUAGE_CHOICES
+from lfc.settings import ORDER_BY_CHOICES
+from lfc.settings import IMAGE_SIZES
+
 
 class Application(models.Model):
-    """
+    """Represents a LFC application.
     """
     name = models.CharField(max_length=100, unique=True)
 
+
 class WorkflowStatesInformation(models.Model):
     """Stores some information about workflows
 
@@ -81,6 +76,7 @@ class WorkflowStatesInformation(models.Model):
 
         return result
 
+
 class Template(models.Model):
     """A template displays the content of an object.
 
@@ -115,6 +111,7 @@ class Template(models.Model):
     def __unicode__(self):
         return self.name
 
+
 class ContentTypeRegistration(models.Model):
     """Stores all registration relevant information of a registered content
     type.
@@ -182,6 +179,7 @@ class ContentTypeRegistration(models.Model):
         """
         return self.templates.all()
 
+
 class Portal(models.Model, PermissionBase):
     """A portal is the root of all content objects. Stores global images and
     some general data about the site.
@@ -215,10 +213,10 @@ class Portal(models.Model, PermissionBase):
 
     """
     title = models.CharField(_(u"Title"), blank=True, max_length=100)
-    standard = models.ForeignKey("BaseContent", verbose_name = _(u"Page"), blank=True, null=True)
+    standard = models.ForeignKey("BaseContent", verbose_name=_(u"Standard"), blank=True, null=True)
 
     from_email = models.EmailField(_(u"From e-mail address"))
-    notification_emails  = models.TextField(_(u"Notification email addresses"))
+    notification_emails = models.TextField(_(u"Notification email addresses"))
     allow_comments = models.BooleanField(_(u"Allow comments"), default=False)
 
     images = generic.GenericRelation("Image", verbose_name=_(u"Images"),
@@ -230,77 +228,100 @@ class Portal(models.Model, PermissionBase):
     def __unicode__(self):
         return self.title
 
+    @property
+    def content_type(self):
+        """
+        Returns the content type of the portal in order to be consistent with
+        BaseContent.
+        """
+        return u"portal"
+
+    def get_content_type(self):
+        """Readable / Displayable content type
+        """
+        return u"Portal"
+
     def get_absolute_url(self):
-        """Returns the absolute url of the portal. It takes the current
-        language into account.
+        """
+        Returns the absolute url of the portal. It takes the current language
+        into account.
         """
         language = translation.get_language()
         if language == settings.LANGUAGE_CODE:
             return reverse("lfc_base_view")
         else:
-            return reverse("lfc_base_view", kwargs={"language" : language})
+            return reverse("lfc_base_view", kwargs={"language": language})
 
     def get_notification_emails(self):
-        """Returns the notification e-mail addresses as list.
+        """
+        Returns the notification e-mail addresses as list.
         """
         adresses = re.split("[\s,]+", self.notification_emails)
         return adresses
 
     def are_comments_allowed(self):
-        """Returns whether comments are allowed globally or not.
+        """
+        Returns whether comments are allowed globally or not.
         """
         return self.allow_comments
 
     def get_parent_for_permissions(self):
-        """Fullfills the contract of django-permissions. Returns just None as
-        there is no parent for portlets.
+        """
+        Fullfills the contract of django-permissions. Returns just None as there
+        is no parent for portlets.
         """
         return None
 
     def get_parent_for_portlets(self):
-        """Fullfills the contract of django-portlets. Returns just None as
-        there is no parent for portlets.
+        """
+        Fullfills the contract of django-portlets. Returns just None as there
+        is no parent for portlets.
         """
         return None
 
     def get_template(self):
-        """Returns the current template of the portal.
+        """
+        Returns the current template of the portal.
         """
         # TODO: Define default template in portal
         return Template.objects.get(name="Article")
 
     def get_children(self, request=None, *args, **kwargs):
-        """Returns the children of the portal. If the request is passed the
+        """
+        Returns the children of the portal. If the request is passed the
         permissions of the current user is taken into account. Additionally
         other valid filters can be passed, e.g. slug = "page-1".
         """
         return lfc.utils.get_content_objects(request, parent=None, **kwargs)
 
     def has_permission(self, user, codename):
-        """Overwrites django-permissions' has_permission in order to add LFC
+        """
+        Overwrites django-permissions' has_permission in order to add LFC
         specific groups.
         """
         # Every user is also anonymous user
         try:
-            roles = [Role.objects.get(name="Anonymous")]
+            roles = [lfc.utils.get_cached_object(Role, name="Anonymous").id]
         except Role.DoesNotExist:
             roles = []
 
         # Check whether the current user is the creator of the current object.
         try:
             if user == self.creator:
-                roles.append(Role.objects.get(name="Owner"))
+                roles.append(Role.objects.get(name="Owner").id)
         except (AttributeError, Role.DoesNotExist):
             pass
 
         return super(Portal, self).has_permission(user, codename, roles)
 
     def check_permission(self, user, codename):
-        """Overwrites django-permissions' check_permission in order to add LFC
+        """
+        Overwrites django-permissions' check_permission in order to add LFC
         specific groups.
         """
         if not self.has_permission(user, codename):
-            raise Unauthorized("%s doesn't have permission %s for portal" % (user, codename))
+            raise Unauthorized("'%s' doesn't have permission '%s' for portal." % (user, codename))
+
 
 class AbstractBaseContent(models.Model, WorkflowBase, PermissionBase):
     """The root of all content types. It provides the inheritable
@@ -317,6 +338,7 @@ class AbstractBaseContent(models.Model, WorkflowBase, PermissionBase):
     class Meta:
         abstract = True
 
+
 class BaseContent(AbstractBaseContent):
     """Base content object. From this class all content types should inherit.
     It should never be instantiated.
@@ -327,14 +349,14 @@ class BaseContent(AbstractBaseContent):
         The content type of the specific content object.
 
     title
-        The title of the object. By default this displayed on top of very
-        object within the title tag of the HTML page (together with the portal's
-        title).
+        The title of the object. By default this is displayed on top of every
+        object.
 
     display_title
         Set to false to hide the title within the HTML of the object. This can
         be helpful to provide a custom title within the text field of an
         object.
+
     slug
         The part of URL within the parent object. By default the absolute URL
         of an object is created by all involved content objects.
@@ -370,6 +392,10 @@ class BaseContent(AbstractBaseContent):
         selected out of the children of the object. If there is one, this is
         displayed instead of the object itself.
 
+    order_by
+        Defines how the children of the object are ordered (default is the
+        position).
+
     exclude_from_navigation
         If set to True, the object is not displayed within the navigation (top
         tabs and navigation tree).
@@ -396,6 +422,10 @@ class BaseContent(AbstractBaseContent):
         if given the object is only public when the end date is not reached
         yet.
 
+    meta_title
+        The meta title of the page. This is displayed within the title tag of
+        the rendered HTML.
+
     meta_keywords
         The meta keywords of the object. This is displayed within the meta
         keywords tag of the rendered HTML.
@@ -418,7 +448,6 @@ class BaseContent(AbstractBaseContent):
         The content which is searched for this object. This attribute should
         not get directly. Rather the get_searchable_text method should be used.
     """
-
     content_type = models.CharField(_(u"Content type"), max_length=100, blank=True)
 
     title = models.CharField(_(u"Title"), max_length=100)
@@ -438,6 +467,7 @@ class BaseContent(AbstractBaseContent):
     parent = models.ForeignKey("self", verbose_name=_(u"Parent"), blank=True, null=True, related_name="children")
     template = models.ForeignKey("Template", verbose_name=_(u"Template"), blank=True, null=True)
     standard = models.ForeignKey("self", verbose_name=_(u"Standard"), blank=True, null=True)
+    order_by = models.CharField(_(u"Order by"), max_length=20, default="position", choices=ORDER_BY_CHOICES)
 
     exclude_from_navigation = models.BooleanField(_(u"Exclude from navigation"), default=False)
     exclude_from_search = models.BooleanField(_(u"Exclude from search results"), default=False)
@@ -449,6 +479,7 @@ class BaseContent(AbstractBaseContent):
     start_date = models.DateTimeField(_(u"Start date"), null=True, blank=True)
     end_date = models.DateTimeField(_(u"End date"), null=True, blank=True)
 
+    meta_title = models.CharField(_(u"Meta title"), max_length=100, default="<portal_title> - <title>")
     meta_keywords = models.TextField(_(u"Meta keywords"), blank=True, default="<tags>")
     meta_description = models.TextField(_(u"Meta description"), blank=True, default="<description>")
 
@@ -470,15 +501,42 @@ class BaseContent(AbstractBaseContent):
     def __unicode__(self):
         return unicode(self.title)
 
-    def save(self, force_insert=False, force_update=False):
-        """Djangos default save method. This is overwritten to do some LFC
-        related stuff if a content object is saved.
+    def has_meta_data_tab(self):
+        return getattr(settings, "LFC_MANAGE_META_DATA", True)
+
+    def has_children_tab(self):
+        return getattr(settings, "LFC_MANAGE_CHILDREN", True)
+
+    def has_images_tab(self):
+        return getattr(settings, "LFC_MANAGE_IMAGES", True)
+
+    def has_files_tab(self):
+        return getattr(settings, "LFC_MANAGE_FILES", True)
+
+    def has_portlets_tab(self):
+        return getattr(settings, "LFC_MANAGE_PORTLETS", True)
+
+    def has_comments_tab(self):
+        return getattr(settings, "LFC_MANAGE_COMMENTS", True)
+
+    def has_seo_tab(self):
+        return getattr(settings, "LFC_MANAGE_SEO", True)
+
+    def has_permissions_tab(self):
+        return getattr(settings, "LFC_MANAGE_PERMISSIONS", True)
+
+    def get_tabs(self, request):
+        return []
+
+    def save(self, *args, **kwargs):
+        """Django's default save method. This is overwritten to do some LFC
+        related stuff when a content object is saved.
         """
         self.searchable_text = self.get_searchable_text()
         if self.content_type == "":
             self.content_type = self.__class__.__name__.lower()
 
-        super(BaseContent, self).save()
+        super(BaseContent, self).save(*args, **kwargs)
 
         # Set the initial state if there is none yet
         co = self.get_content_object()
@@ -501,25 +559,31 @@ class BaseContent(AbstractBaseContent):
 
         slugs.reverse()
 
-        slug =  "/".join(slugs)
+        slug = "/".join(slugs)
 
         if page.language == settings.LANGUAGE_CODE:
-            return ("lfc_base_view", (), {"slug" : slug})
+            return ("lfc_base_view", (), {"slug": slug})
         elif page.language == "0":
-            language = translation.get_language()
-            if language == settings.LANGUAGE_CODE:
-                return ("lfc_base_view", (), {"slug" : slug})
+            if page.parent:
+                language = page.parent.language
+                if language == "0":
+                    return ("lfc_base_view", (), {"slug": slug})
             else:
-                return ("lfc_base_view", (), {"slug" : slug, "language" : language})
+                language = translation.get_language()
+
+            if language == settings.LANGUAGE_CODE:
+                return ("lfc_base_view", (), {"slug": slug})
+            else:
+                return ("lfc_base_view", (), {"slug": slug, "language": language})
         else:
-            return ("lfc_base_view", (), {"slug" : slug, "language" : page.language})
+            return ("lfc_base_view", (), {"slug": slug, "language": page.language})
 
     get_absolute_url = models.permalink(get_absolute_url)
 
     def get_content_object(self):
         """Returns the specific content object of the instance. This method
         can be called if one has a BaseContent and want the specific content
-        type e.g. Page.
+        type.
         """
         # TODO: Ugly but works. There must be a cleaner way. isinstance doesn't
         # work of course.
@@ -528,11 +592,6 @@ class BaseContent(AbstractBaseContent):
         else:
             return self
 
-    def get_content_type(self):
-        """Returns the content type as string.
-        """
-        return self.__class__.__name__
-
     def get_searchable_text(self):
         """Returns the searchable text of this content type. By default it
         takes the title the description of the instance into account. Sub
@@ -541,11 +600,16 @@ class BaseContent(AbstractBaseContent):
         result = self.title + " " + self.description
         return result.strip()
 
-    def form(self, **kwargs):
-        """Returns the add/edit form for the object. This method has to be
-        overwritten and implemented by sub classes.
+    def edit_form(self, **kwargs):
+        """Returns the edit form for the object.
         """
-        raise NotImplementedError, "form has to be implemented by sub classed"
+        raise(NotImplementedError, "form has to be implemented by sub classed")
+
+    def add_form(self, **kwargs):
+        """Returns the add/edit form for the object.
+        """
+        from lfc.manage.forms import AddForm
+        return AddForm(**kwargs)
 
     def get_ancestors(self):
         """Returns all ancestors of a content object.
@@ -567,12 +631,10 @@ class BaseContent(AbstractBaseContent):
         return ancestors
 
     def get_content_type(self):
+        """Returns the content type of the object.
         """
-        """
-        try:
-            return ContentTypeRegistration.objects.get(type=self.content_type).name
-        except ContentTypeRegistration.DoesNotExist:
-            return _(u"n/a")
+        info = lfc.utils.registration.get_info(self.content_type)
+        return info.name
 
     def get_descendants(self, request=None, result=None):
         """Returns all descendants of the content object. If the request is
@@ -586,6 +648,13 @@ class BaseContent(AbstractBaseContent):
 
         return result
 
+    def has_children(self, request=None, *args, **kwargs):
+        """Returns True if the object has children. If the request is
+        passed the permissions of the current user is taken into account.
+        Other valid filters can be passed also, e.g. slug = "page-1".
+        """
+        return len(lfc.utils.get_content_objects(request, parent=self, **kwargs)) > 0
+
     def get_children(self, request=None, *args, **kwargs):
         """Returns the children of the content object. If the request is
         passed the permissions of the current user is taken into account.
@@ -603,6 +672,14 @@ class BaseContent(AbstractBaseContent):
         except IndexError:
             return None
 
+    def get_meta_title(self):
+        """Returns the meta title of the instance. Replaces some placeholders
+        with the according content.
+        """
+        title = self.meta_title.replace("<title>", self.title)
+        title = title.replace("<portal_title>", lfc.utils.get_portal().title)
+        return title
+
     def get_meta_keywords(self):
         """Returns the meta keywords of the instance. Replaces some
         placeholders with the according content.
@@ -627,9 +704,9 @@ class BaseContent(AbstractBaseContent):
         if self.template is not None:
             return self.template
         else:
-            template = lfc.utils.registration.get_default_template(self)
-            if template is not None:
-                return template
+            default_template = lfc.utils.registration.get_default_template(self)
+            if default_template is not None:
+                return default_template
             else:
                 return lfc.utils.get_portal().get_template()
 
@@ -654,7 +731,7 @@ class BaseContent(AbstractBaseContent):
             if self.canonical:
                 obj = BaseContent.objects.get(pk=self.canonical.id)
                 if self.has_permission(request.user, "view"):
-                    return obj
+                    return obj.get_content_object()
                 else:
                     return None
 
@@ -728,6 +805,93 @@ class BaseContent(AbstractBaseContent):
         """
         return self.parent and self.parent.get_content_object() or lfc.utils.get_portal()
 
+    def set_context(self, request):
+        self.context = self.get_context(request)
+
+    def get_context(self, request):
+        """Calculates and returns the request context.
+        """
+        obj_template = self.get_template()
+
+        # Children
+        # CACHE
+        children_cache_key = "%s-children-%s-%s-%s" % \
+            (settings.CACHE_MIDDLEWARE_KEY_PREFIX, self.content_type, self.id, request.user.id)
+        sub_objects = cache.get(children_cache_key)
+        if sub_objects is None:
+            # Get sub objects (as LOL if requested)
+            if obj_template.children_columns == 0:
+                sub_objects = self.get_children(request)
+            else:
+                sub_objects = lfc.utils.getLOL(self.get_children(request), obj_template.children_columns)
+
+            cache.set(children_cache_key, sub_objects)
+
+        # Images
+        # CACHE
+        images_cache_key = "%s-images-%s-%s" % (settings.CACHE_MIDDLEWARE_KEY_PREFIX, self.content_type, self.id)
+        cached_images = cache.get(images_cache_key)
+        if cached_images:
+            image = cached_images["image"]
+            images = cached_images["images"]
+            subimages = cached_images["subimages"]
+        else:
+            temp_images = list(self.images.all())
+            if temp_images:
+                if obj_template.images_columns == 0:
+                    images = temp_images
+                    image = images[0]
+                    subimages = temp_images[1:]
+                else:
+                    images = lfc.utils.getLOL(temp_images, obj_template.images_columns)
+                    subimages = lfc.utils.getLOL(temp_images[1:], obj_template.images_columns)
+                    image = images[0][0]
+            else:
+                image = None
+                images = []
+                subimages = []
+
+            cache.set(images_cache_key, {
+                "image": image,
+                "images": images,
+                "subimages": subimages
+            })
+
+        self.context = RequestContext(request, {
+            "lfc_context": self,
+            "self": self,
+            "images": images,
+            "image": image,
+            "subimages": subimages,
+            "files": self.files.all(),
+            "sub_objects": sub_objects,
+            "portal": lfc.utils.get_portal(),
+        })
+
+        return self.context
+
+    def render(self, request):
+        """Renders the object content.
+        """
+        if self.context is None:
+            self.context = self.get_context()
+
+        # CACHE
+        template_cache_key = "%s-template-%s-%s" % (settings.CACHE_MIDDLEWARE_KEY_PREFIX, self.content_type, self.id)
+        obj_template = cache.get(template_cache_key)
+        if obj_template is None:
+            obj_template = self.get_template()
+            cache.set(template_cache_key, obj_template)
+
+        tags = ""
+        for tag in getattr(settings, "LFC_TAGS", []):
+            tags += "{%% load %s %%}" % tag
+
+        # Render twice. This makes tags within text / short_text possible.
+        result = render_to_string(obj_template.path, self.context)
+        result = template.Template(tags + " " + result).render(self.context)
+        return result
+
     # django-permissions
     def get_parent_for_permissions(self):
         """Returns the parent from which permissions are inherited. The
@@ -739,12 +903,10 @@ class BaseContent(AbstractBaseContent):
         """Overwrites django-permissions' has_permission in order to add LFC
         specific groups.
         """
-        # CACHE
-        cache_key = "%s-object-%s-%s-%s" % \
-              (settings.CACHE_MIDDLEWARE_KEY_PREFIX, self.id, user.id, codename)
-        result = cache.get(cache_key)
-        if result:
-            return result
+        try:
+            return user.permissions[str(self.id)][codename]
+        except:
+            pass
 
         # Every user is also anonymous user
         try:
@@ -761,8 +923,14 @@ class BaseContent(AbstractBaseContent):
 
         result = super(BaseContent, self).has_permission(user, codename, roles)
 
-        # set cache
-        cache.set(cache_key, result)
+        if not getattr(user, "permissions", False):
+            user.permissions = {}
+
+        if not user.permissions.get(str(self.id), False):
+            user.permissions[str(self.id)] = {}
+
+        user.permissions[str(self.id)][codename] = result
+
         return result
 
     def check_permission(self, user, codename):
@@ -770,7 +938,7 @@ class BaseContent(AbstractBaseContent):
         specific groups.
         """
         if not self.has_permission(user, codename):
-            raise Unauthorized("%s doesn't have permission %s for object %s" % (user, codename, self.slug))
+            raise Unauthorized("'%s' doesn't have permission '%s' for object '/%s' (%s)." % (user, codename, self.slug, self.__class__.__name__))
 
     def is_active(self, user):
         """Returns True if now is between start and end date of the object.
@@ -810,41 +978,19 @@ class BaseContent(AbstractBaseContent):
         for transition in state.transitions.all():
             permission = transition.permission
             if permission is None or self.has_permission(user, permission.codename):
-               transitions.append(transition)
+                transitions.append(transition)
 
         return transitions
 
-class Page(BaseContent):
-    """A page is the foremost object within lfc which shows information to the
-    user.
-
-    **Attributes**:
-
-    text:
-        The main text of the page.
-    """
-    text = models.TextField(_(u"Text"), blank=True)
-
-    def get_searchable_text(self):
-        """Returns the searchable text of the page. This adds the text to the 
-        default searchable text.
-        """
-        result = self.title + " " + self.description + " " + self.text
-        return result.strip()
-
-    def form(self, **kwargs):
-        """Returns the add/edit form of the page.
-        """
-        from lfc.manage.forms import CoreDataForm
-        return CoreDataForm(**kwargs)
 
 class Image(models.Model):
     """An image which can be displayes within HTML. Generates automatically
     various sizes.
 
     title
-        The title of the image. Used within the title and alt tag
-        of the image
+        The title of the image. Used within the title and alt tag of the
+        image.
+
     slug
         The URL of the image
 
@@ -857,31 +1003,25 @@ class Image(models.Model):
     caption
         The caption of the image. Can be used within the content (optional)
 
-    short_description
-        A short description of the image. Can be used within the content
-        (optional)
-
     description
-        A long description of the image. Can be used within the content
+        A description of the image. Can be used within the content
         (optional)
 
     image
         The image file.
     """
-    title = models.CharField(blank=True, max_length=100)
-    slug = models.SlugField()
+    title = models.CharField(_(u"Title"), blank=True, max_length=100)
+    slug = models.SlugField(_(u"Slug"),)
 
     content_type = models.ForeignKey(ContentType, verbose_name=_(u"Content type"), related_name="images", blank=True, null=True)
     content_id = models.PositiveIntegerField(_(u"Content id"), blank=True, null=True)
     content = generic.GenericForeignKey(ct_field="content_type", fk_field="content_id")
 
-    position = models.SmallIntegerField(default=999)
-    caption = models.CharField(blank=True, max_length=100)
-    short_description = models.TextField(blank=True)
-    description = models.TextField(blank=True)
+    position = models.SmallIntegerField(_(u"Position"), default=999)
+    caption = models.CharField(_(u"Caption"), blank=True, max_length=100)
+    description = models.TextField(_(u"Description"), blank=True)
     creation_date = models.DateTimeField(_(u"Creation date"), auto_now_add=True)
-    image = ImageWithThumbsField(_(u"Image"), upload_to="uploads",
-        sizes=((60, 60), (100, 100), (200, 200), (400, 400), (600, 600), (800, 800)))
+    image = ImageWithThumbsField(_(u"Image"), upload_to="uploads", sizes=IMAGE_SIZES)
 
     class Meta:
         ordering = ("position", )
@@ -890,29 +1030,29 @@ class Image(models.Model):
         return self.title
 
     def get_absolute_url(self):
-        return ("gallery.views.photo", (), {"slug" : self.slug})
+        return ("gallery.views.photo", (), {"slug": self.slug})
     get_absolute_url = models.permalink(get_absolute_url)
 
+
 class File(models.Model):
     """A downloadable file.
 
     **Attributes:**
 
     title
-        The title of the image. Used within the title and alt tag
-        of the image.
+        The title of the file.
 
     slug
-        The URL of the image.
+        The URL of the file.
 
     content
         The content object the file belongs to (optional).
 
     position
-        The ordinal number within the content object. Used to order the images.
+        The ordinal number within the content object. Used to order the files.
 
     description
-        A long description of the image. Can be used within the content
+        A long description of the file. Can be used within the content
         (optional).
 
     file
@@ -926,7 +1066,7 @@ class File(models.Model):
     content = generic.GenericForeignKey(ct_field="content_type", fk_field="content_id")
 
     position = models.SmallIntegerField(default=999)
-    description = models.CharField(blank=True, max_length=100)
+    description = models.TextField(blank=True)
     creation_date = models.DateTimeField(_(u"Creation date"), auto_now_add=True)
     file = models.FileField(upload_to="files")
 
@@ -937,169 +1077,4 @@ class File(models.Model):
         return self.title
 
     def get_absolute_url(self):
-        return reverse("lfc_file", kwargs={"id" : self.id})
-
-#### Portlets
-###############################################################################
-
-class NavigationPortlet(Portlet):
-    """A portlet to display the navigation tree.
-
-    Note: this reuses mainly the navigation inclusion tag.
-
-    Parameters:
-
-        - start_level:
-            The tree is displayed from this level 1. The tree starts with 1
-
-        - expand_level:
-            The tree is expanded up this level. Default is 0, which means the
-            tree is not expanded at all but the current node.
-    """
-    start_level = models.PositiveSmallIntegerField(default=1)
-    expand_level = models.PositiveSmallIntegerField(default=0)
-
-    def render(self, context):
-        """Renders the portlet as HTML.
-        """
-        request = context.get("request")
-        return render_to_string("lfc/portlets/navigation_portlet.html", RequestContext(request, {
-            "start_level" : self.start_level,
-            "expand_level" : self.expand_level,
-            "title" : self.title,
-        }))
-
-    def form(self, **kwargs):
-        """
-        """
-        return NavigationPortletForm(instance=self, **kwargs)
-
-class NavigationPortletForm(forms.ModelForm):
-    """Add/edit form for the navigation portlet.
-    """
-    class Meta:
-        model = NavigationPortlet
-
-# TODO: Rename as it is able to display all content types. ContentPortlet, DocumentPortlet, ...?
-class PagesPortlet(Portlet):
-    """A portlet to display arbitrary objects. The objects can be selected by
-    tags.
-
-    **Attributes:**
-
-    limit:
-        The amount of objects which are displayed at maximum.
-
-    tags:
-        The tags an object must have to be displayed.
-    """
-    limit = models.PositiveSmallIntegerField(default=5)
-    tags = models.CharField(blank=True, max_length=100)
-
-    def __unicode__(self):
-        return "%s" % self.id
-
-    def render(self, context):
-        """Renders the portlet as HTML.
-        """
-        objs = BaseContent.objects.filter(
-            language__in=("0", translation.get_language()))
-
-        if self.tags:
-            objs = tagging.managers.ModelTaggedItemManager().with_all(self.tags, objs)[:self.limit]
-
-        return render_to_string("lfc/portlets/pages_portlet.html", {
-            "title" : self.title,
-            "objs" : objs,
-        })
-
-    def form(self, **kwargs):
-        """Returns the add/edit form of the portlet.
-        """
-        return PagesPortletForm(instance=self, **kwargs)
-
-class PagesPortletForm(forms.ModelForm):
-    """Add/edit form of the pages portlet.
-    """
-    tags = TagField(widget=AutoCompleteTagInput(), required=False)
-
-    class Meta:
-        model = PagesPortlet
-
-class RandomPortlet(Portlet):
-    """A portlet to display random objects. The objects can be selected by
-    tags.
-
-    **Attributes:**
-
-    limit:
-        The amount of objects which are displayed at maximum.
-
-    tags:
-        The tags an object must have to be displayed.
-    """
-    limit = models.PositiveSmallIntegerField(default=1)
-    tags = models.CharField(blank=True, max_length=100)
-
-    def render(self, context):
-        """Renders the portlet as HTML.
-        """
-        items = BaseContent.objects.filter(
-            language__in=("0", translation.get_language()))
-
-        if self.tags:
-            items = tagging.managers.ModelTaggedItemManager().with_all(self.tags, items)[:self.limit]
-
-        items = list(items)
-        random.shuffle(items)
-
-        return render_to_string("lfc/portlets/random_portlet.html", {
-            "title" : self.title,
-            "items" : items[:self.limit],
-        })
-
-    def form(self, **kwargs):
-        """Returns the form of the portlet.
-        """
-        return RandomPortletForm(instance=self, **kwargs)
-
-class RandomPortletForm(forms.ModelForm):
-    """Add/Edit form for the random portlet.
-    """
-    tags = TagField(widget=AutoCompleteTagInput(), required=False)
-
-    class Meta:
-        model = RandomPortlet
-
-class TextPortlet(Portlet):
-    """A portlet to display arbitrary HTML text.
-
-    **Attributes:**
-
-    text:
-        The HTML text which is displayed. Can contain any HTML text.
-    """
-
-    text = models.TextField(_(u"Text"), blank=True)
-
-    def __unicode__(self):
-        return "%s" % self.id
-
-    def render(self, context):
-        """Renders the portlet as HTML.
-        """
-        return render_to_string("lfc/portlets/text_portlet.html", {
-            "title" : self.title,
-            "text" : self.text
-        })
-
-    def form(self, **kwargs):
-        """
-        """
-        return TextPortletForm(instance=self, **kwargs)
-
-class TextPortletForm(forms.ModelForm):
-    """Add/Edit form for the text portlet.
-    """
-    class Meta:
-        model = TextPortlet
\ No newline at end of file
+        return reverse("lfc_file", kwargs={"id": self.id})
