@@ -1,4 +1,5 @@
 from copy_tests import *
+from general_tests import *
 from manager_tests import *
 from page_tests import *
 from permission_tests import *
@@ -7,5 +8,5 @@ from portlets_tests import *
 from registration_tests import *
 from utils_tests import *
 
-from permissions.tests import *
-from workflows.tests import *
\ No newline at end of file
+# from permissions.tests import *
+# from workflows.tests import *
