@@ -7,12 +7,16 @@ import permissions.utils
 
 # lfc imports
 import lfc.utils
-from lfc.manage.forms import CoreDataForm
+from lfc.utils import import_module
+from lfc.models import Application
 from lfc.models import BaseContent
-from lfc.models import Page
 from lfc.models import Portal
 from lfc.tests.utils import create_request
 
+# lfc_page imports
+from lfc_page.forms import PageDataForm
+from lfc_page.models import Page
+
 class PageTestCase(TestCase):
     """Tests for Page related stuff.
     """
@@ -22,12 +26,26 @@ class PageTestCase(TestCase):
         from lfc.utils.initialize import initialize
         initialize()
 
+        import_module("lfc_page").install()
+        try:
+            Application.objects.create(name="lfc_page")
+        except Application.DoesNotExist:
+            pass
+
         self.p = Portal.objects.create()
         self.p1 = Page.objects.create(title="Page 1", slug="page-1")
         self.p11 = Page.objects.create(title="Page 1-1", slug="page-1-1", parent=self.p1)
         self.p111 = Page.objects.create(title="Page 1-1-1", slug="page-1-1-1", parent=self.p11)
         self.p12 = Page.objects.create(title="Page 1-2", slug="page-1-2", parent=self.p1)
 
+        # for i18n tests
+        self.p2en = Page.objects.create(title="Page 2 (en)", slug="page-2",
+                                        language='en')
+        self.p2de = Page.objects.create(title="Page 2 (de)", slug="page-2",
+                                      language='de', canonical=self.p2en)
+        self.p2fr = Page.objects.create(title="Page 2 (fr)", slug="page-2",
+                                        language='fr', canonical=self.p2en)
+
         self.anonymous = permissions.utils.register_role("Anonymous")
         self.permission = permissions.utils.register_permission("View", "view")
 
@@ -87,8 +105,8 @@ class PageTestCase(TestCase):
     def test_get_form(self):
         """
         """
-        form = self.p1.form()
-        self.assertEqual(form.__class__, CoreDataForm)
+        form = self.p1.edit_form()
+        self.assertEqual(form.__class__, PageDataForm)
 
     def test_get_ancestors(self):
         """
@@ -243,4 +261,67 @@ class PageTestCase(TestCase):
         self.assertEqual(len(children), 1)
 
         children = self.p1.get_children(request, slug="page-1-2")
-        self.assertEqual(len(children), 0)
\ No newline at end of file
+        self.assertEqual(len(children), 0)
+
+    # i18n tests
+    def test_i18n_has_language(self):
+        request = create_request()
+
+        self.assertEqual(self.p2de.has_language(request, "de"), True)
+        self.assertEqual(self.p2fr.has_language(request, "fr"), True)
+
+        self.assertEqual(self.p2en.has_language(request, "en"), True)
+        self.assertEqual(self.p2en.has_language(request, "de"), True)
+        self.assertEqual(self.p2en.has_language(request, "fr"), True)
+        self.assertEqual(self.p2en.has_language(request, "es"), False)
+
+    def test_i18n_get_canonical(self):
+        request = create_request()
+        self.assertEqual(self.p2de.get_canonical(request), self.p2en)
+        self.assertEqual(self.p2en.get_canonical(request), self.p2en)
+        self.assertEqual(self.p2fr.get_canonical(request), self.p2en)
+
+    def test_i18n_get_translation(self):
+        request = create_request()
+        self.assertEqual(self.p2en.is_translation(), False)
+        self.assertEqual(self.p2de.is_translation(), True)
+        self.assertEqual(self.p2fr.is_translation(), True)
+
+    def test_i18n_get_translation2(self):
+        request = create_request()
+        translation = self.p2en.get_translation(request, 'en')
+        if translation:
+            self.assertEqual(translation, self.p2en)
+        translation = self.p2en.get_translation(request, 'fr')
+        if translation:
+            self.assertEqual(translation, self.p2fr)
+        translation = self.p2en.get_translation(request, 'de')
+        if translation:
+            self.assertEqual(translation, self.p2de)
+
+    def test_i18n_get_translation3(self):
+        request = create_request()
+        translation = self.p2de.canonical.get_translation(request, 'en')
+        if translation:
+            self.assertEqual(translation, self.p2en)
+        translation = self.p2fr.canonical.get_translation(request, 'en')
+        if translation:
+            self.assertEqual(translation, self.p2en)
+        translation = self.p2fr.canonical.get_translation(request, 'de')
+        if translation:
+            self.assertEqual(translation, self.p2de)
+
+
+    def test_i18n_view_set_language(self):
+        request = create_request()
+        from lfc.views import set_language
+
+        self.assertEqual(set_language(request, 'en', self.p2en.id)['Location'],
+                         '/page-2')
+        self.assertEqual(set_language(request, 'de', self.p2en.id)['Location'],
+                         '/de/page-2')
+        self.assertEqual(set_language(request, 'fr', self.p2en.id)['Location'],
+                         '/fr/page-2')
+
+        self.assertEqual(set_language(request, 'fr', self.p2de.id)['Location'],
+                         '/fr/page-2')
